/* $Id: iss_mac.h 606 2008-08-15 08:39:26Z gloomy $ */

#ifndef _ISS_MAC_H_INCLUDED
#define _ISS_MAC_H_INCLUDED

#include <stdio.h>		/* For FILE  */
#include <sys/types.h>		/* For pid_t */

#include <parsec/parsec.h>
#include <parsec/parsec_mac.h>
#include <parsec/mac_db.h>
#include <parsec/aux.h>
#include <linux/mac.h>		/* for mac   */
#include <pwd.h>
#include <sys/stat.h>

/*
#define MAC_MAX_LEV          MAC_MAX_LEVEL
#define MAC_MAX_CAT          MAC_MAX_CATEGORY
*/

#ifdef __cplusplus
extern "C" {
#endif

/***************************************************
 *           WORKING WITH CONFIG FILES             *
 ***************************************************/

#define MAC_DEFAULT_FILE      "/etc/security/mac"
#define MAC_DEFAULT_LEV_FILE  "/etc/security/mac_levels"
#define MAC_DEFAULT_CAT_FILE  "/etc/security/mac_categories"

/* For compatibility with <path.h>.
 * You can use these definitions, or definitions
 * above, without difference.
 */
#define _PATH_MAC            MAC_DEFAULT_FILE
#define _PATH_MAC_LEVELS     MAC_DEFAULT_LEV_FILE
#define _PATH_MAC_CATEGORIES MAC_DEFAULT_CAT_FILE
/*
#define MAKE_SECRET_CATALOGS
#define SECURITY "security"
*/
/* #define WORDS_LEVELS_CATEGORIES */

/*	typedef __u8 mac_lev_t;
	typedef __u64 mac_cat_t;

	struct mac_label {
		mac_lev_t lev;
		mac_cat_t cat;
	};

	struct mac_user {
		char *name;
		struct mac_label min;
		struct mac_label max;
	};

	struct mac_lev {
		char *name;
		mac_lev_t value;
	};

	struct mac_cat {
		char *name;
		mac_cat_t value;
	};
*/
/* /etc/security/mac: */
/*	extern int setmacent(void);
	extern void endmacent(void);
	extern struct mac_user *getmacent(void);

	extern struct mac_user *getmacnam(const char *name);
	extern struct mac_user *getmacuid(uid_t uid);
	extern int putmacent(const struct mac_user *mu);

	extern struct mac_user *fgetmacent(FILE * f);
	extern void fputmacent(FILE * f, const struct mac_user *mu);

	extern int rmmacnam(const char *name);
	extern int make_secret_catalogs(struct mac_user *mu,
					struct passwd *pswd);
*/
/* /etc/security/mac_levels: */
/*	extern int setlevent(void);
	extern void endlevent(void);
	extern struct mac_lev *getlevent(void);

	extern struct mac_lev *getlevnam(const char *name);
	extern struct mac_lev *getlevval(mac_lev_t value);
	extern int putlevent(const struct mac_lev *ml);

	extern struct mac_lev *fgetlevent(FILE * f);
	extern void fputlevent(FILE * f, const struct mac_lev *ml);

	extern int rmlevnam(const char *name);
	extern int rmlevval(mac_lev_t value);
*/
/* /etc/security/mac_categories: */
/*	extern int setcatent(void);
	extern void endcatent(void);
	extern struct mac_cat *getcatent(void);

	extern struct mac_cat *getcatnam(const char *name);
	extern struct mac_cat *getcatval(mac_cat_t value);
	extern int putcatent(const struct mac_cat *mc);

	extern struct mac_cat *fgetcatent(FILE * f);
	extern void fputcatent(FILE * f, const struct mac_cat *mc);

	extern int rmcatnam(const char *name);
	extern int rmcatval(mac_cat_t value);
*/
/***************************************************
 *           WORKING WITH PROCESSES                *
 ***************************************************/
	extern int setmac(const struct mac_label *mac);
	extern int getmac(struct mac_label *buff);
	extern int psetmac(pid_t pid, const struct mac_label *mac);
	extern int pgetmac(pid_t pid, struct mac_label *buff);

/***************************************************
 *           WORKING WITH FILE SYSTEM              *
 ***************************************************/
	typedef uint32_t mac_attr_t;
/* FILES: */
	extern int chmac(const char *path, const struct mac_label *mac,
			 mac_attr_t attr);
	extern int fchmac(int fd, const struct mac_label *mac, mac_attr_t attr);
	extern int statmac(const char *path, struct mac_label *mbuff,
			   mac_attr_t * abuff);
	extern int fstatmac(int fd, struct mac_label *mbuff,
			    mac_attr_t * abuff);

/* SOCKETS: */
	extern int schmac(int s, const struct mac_label *mac);
	extern int schpriv(int s, int yes);
	extern int sgetmac(int s, struct mac_label *mbuff);
	extern int sgetpriv(int s, int *yesbuff);

/* some mcbc specific defines from sys/socket.h (asm/socket.h) */
#define SOL_MCBC        0x8000	/* for mcbc */
#define SO_LABEL        0x70	/* for mcbc */
#define SO_PRIV         0x71	/* for mcbc */

/***************************************************
 *           AUXILARY FUNCTIONS                    *
 ***************************************************/

/* 
 * Function "strtou64" converts the string "data" to 
 * number of type __u64 according to base "base".
 * Supplied base 2, 10 and 16 only. 
 * "endptr" points to first non-convertable character:
 */
static inline uint64_t strtou64(const char *data, char **endptr, int base)
{
	return maux_strtou64(data, endptr, base);
}

/* 
 * Function "u64tostr" converts number "data" to string "tostr",
 * according to base "base".
 * Supplied base 2, 10 and 16 only.
 * Returns pointer to result string (&tostr[0]).
 * In case of error NULL returned.
 */
static inline char *u64tostr(char *tostr, uint64_t data, int base)
{
	return maux_u64tostr(tostr, data, base);
}

#ifdef __cplusplus
}
#endif
#endif				/* _ISS_MAC_H_INCLUDED */
